# Copyright (c) 2010-2011 David Love
#
# Permission to use, copy, modify, and/or distribute this software for 
# any purpose with or without fee is hereby granted, provided that the 
# above copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES 
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF 
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR 
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

# @author David Love

module FluxTuna::Witness
  
  # 
  # @abstract Defines the core {Witness} class
  #
  class AbstractWitness
    
    # Default constructor. The names of the shatter, mutate and bind class
    # to use for this {Witness} must be passed in. These will be called in
    # place of the the methods of this class when appropriate.
    def initialize(shatter, mutate, bind)
      shatter_string = "Witness::Shatter::" + shatter.to_s
      @shatter = shatter_string.to_sym
      
      mutate_string = "Witness::Mutate::" + shatter.to_s
      @mutate = mutate_string.to_sym
      
      bind_string = "Witness::Bind::" + shatter.to_s
      @bind = bind_string.to_sym
    end

    # Calls the named shatter class to break the original structure
    def shatter
      call_shatter {}
    end

    # Calls the named bind class to mutate the representation
    def mutate
      call_mutate {}
    end
    
    # Calls the named bind class to create the representation tree
    def bind
      call_bind {}
    end
    
    # Calls the named shatter class to break the original structure
    def call_shatter(&block)
      Object.const_get(@shatter).send(:shatter)
    end

    # Calls the named bind class to mutate the representation
    def call_mutate(&block)
      Object.const_get(@mutate).send(:mutate)
    end
    
    # Calls the named bind class to create the representation tree
    def call_bind(&block)
      Object.const_get(@bind).send(:bind)
    end
    
    # Proxy calls are only used by the derived classes
    private :call_shatter, :call_mutate, :call_bind
    
  end
  
end