# Copyright (c) 2010-2011 David Love
#
# Permission to use, copy, modify, and/or distribute this software for 
# any purpose with or without fee is hereby granted, provided that the 
# above copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES 
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF 
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR 
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

# @author David Love

module FluxTuna::Witness

  # Create a Witness object for the specified directory
  class DirFileWitness < FluxTuna::Witness::AbstractWitness
  
    # Default constructor
    def initialize
      super(:Dir, :Null, :Tree)
      
      @content = FluxTuna::Core::NameTrie.new
    end
  
    # Run the shatter function, taking 
    # +path+ is the {File.glob} shell glob for the path and pattern to match
    # when looking for files
    def shatter(path)
      
      # Walk the path,
      Dir.glob(path){|file_name|
        
        # Ignore directories
        unless File.directory?(file_name) then
          
          # Ignore the special files as well
          unless file_name == "." or file_name == ".." then
            
            # Create a GUID, then add this filename as the descriptor
            @content[UUIDTools::UUID.random_create] = file_name
          end        
        end
        
      }
      
    end
  end
  
end